#ifndef BMPTOOLS_H // this avoids multiple inclusion of this file
#define BMPTOOLS_H

#include "jdct.h"

#ifndef _INC_WINDOWS
// Make sure "Struct member alignment" in the project properties 
// is set to "1 Byte (/Zp1)" if these structs are used!
// The following #pragma will make sure that happens in visual c++
#pragma pack(push, 1)

/*
	The following structures are used when loading BMP images from a file.
	You should only ever need to look at the BITMAPINFOHEADER struct. 
*/
typedef unsigned long       DWORD;
typedef unsigned short      WORD;
typedef long LONG;

/* structures for defining DIBs */
// You probably will not need this struct, it is used by loadbmp internally
typedef struct tagBITMAPFILEHEADER {
        WORD    bfType;
        DWORD   bfSize;
        WORD    bfReserved1;
        WORD    bfReserved2;
        DWORD   bfOffBits;
} BITMAPFILEHEADER;

typedef struct tagBITMAPINFOHEADER{
        DWORD      biSize;				// Size of the header, should be 40
        LONG       biWidth;				// The width of the image in pixels
        LONG       biHeight;			// The height of the image in pixels
        WORD       biPlanes;			// Number of color planes. Should be one
        WORD       biBitCount;			// Number of bits per pixel
        DWORD      biCompression;		// Compression method used in the image
        DWORD      biSizeImage;			// Size of the raw image data in bytes
        LONG       biXPelsPerMeter;		// horizontal resolution in pixels per meter
        LONG       biYPelsPerMeter;		// vertical resolution in pixels per meter
        DWORD      biClrUsed;			// Number of colors in the color palette
        DWORD      biClrImportant;		// Number of important colors
} BITMAPINFOHEADER;

// Magic BMP number
#ifndef BITMAP_ID
#define BITMAP_ID 0x4D42
#endif

// Restore old alignment rules
#pragma pack(pop)
#endif

typedef union {		/* block of pixel-space values */
  int	block[8][8];
  int	linear[64];
} PBlock;

typedef union {		/* block of frequency-space values */
  int block[8][8];
  int linear[64];
} FBlock;

unsigned char *loadbmp(char *filename, BITMAPINFOHEADER *bmpinfo);
void DCT(PBlock* P, FBlock* F);
void IDCT(FBlock *S, PBlock *T);

static const int ZZ[] = {
	0,  1,  8, 16,  9,  2,  3, 10,
	17, 24, 32, 25, 18, 11,  4,  5,
	12, 19, 26, 33, 40, 48, 41, 34,
	27, 20, 13,  6,  7, 14, 21, 28,
	35, 42, 49, 56, 57, 50, 43, 36,
	29, 22, 15, 23, 30, 37, 44, 51,
	58, 59, 52, 45, 38, 31, 39, 46,
	53, 60, 61, 54, 47, 55, 62, 63
};


// Make sure "Struct member alignment" in the project properties 
// is set to "1 Byte (/Zp1)" if these structs are used!
// The following #pragma will make sure that happens in visual c++
#pragma pack(push, 1)

// The following structs van be used when writing you EJPG files
typedef struct tagEJPGHeader{
	int magic_number;
	short unsigned int width;
	short unsigned int height;
	short unsigned int Y_scale;
	short unsigned int Cr_scale;
	short unsigned int Cb_scale;
} EJPGHeader;

typedef struct tagEJPGBlock{
	short signed int DC_value;
	signed char block[63];
} EJPGBlock;


// Restore old alignment rules
#pragma pack(pop)

#endif